!+
! Module: emask_mod
!
! Authors:  Hermann Brunner - AIP
!          
! Description: Detection mask creation (from exposure images), marking
! 
! Comments: 
! 
! $Id: module.f90,v 1.5 1998/01/13 08:39:15 gvacanti Exp $
!- 
module emask_mod

  use dal
  use param  
  use edetect_io  

  implicit none 
  
  private 

  public emask

contains 
   
  !---------------------------------------------------------------------
  subroutine emask
  !---------------------------------------------------------------------
    
    integer :: ix, iy
    real(kind=single) :: fracth, gradth, emax
    character(len=1000) :: ostring
    character(len=1024), dimension(16) :: image
    character(len=16), dimension(16) :: instrume 
    integer, dimension(16) :: band     
    integer, dimension(2) :: npix 
    real(kind=single), dimension(:,:), pointer :: expo
    integer(kind=int32), dimension(:,:), pointer :: x
    integer, dimension(:), pointer :: shape
    
    type(DataSetT) :: set, setO
    type(ArrayT) :: earr, marr
     
! threshold parameter [0.0<parameter<1.0]
         
    fracth = realParameter("threshold1")
    gradth = realParameter("threshold2")

! names of exposure images (A,B,C) one per instrument
! for the moment, read just the first one

!   call get_files ("expo_image", image, instrume, band, .true.)    

    image(1) = stringParameter("expimageset")
    set = dataSet(image(1),READ)
    earr = array(set,"PRIMARY",READ)
    expo => real32Array2Data(earr) 

    shape => dimensions(earr)
    npix(1) = shape(0)
    npix(2) = shape(1)
      
! bbname of detection mask file
         
    ostring=stringParameter("detmaskset")
    setO=dataset(ostring,Create)
    marr=addArray(setO,"PRIMARY",Integer32,shape)
    x => int32Array2Data(marr)
    x = 1 

    emax = expo(0,0)
    do ix = 0,npix(1)-1
      do iy = 0,npix(2)-1
        emax = max(expo(ix,iy),emax)
      end do
    end do

    fracth = fracth*emax
    gradth = gradth*emax

    do ix = 0,npix(1)-1
      do iy = 0,npix(2)-1
        if (expo(ix,iy) < fracth) then
          x(ix,iy) = 0
        end if
      end do
    end do

     do ix = 1,npix(1)-2
       do iy = 1,npix(2)-2
        if (sqrt( ( expo(ix+1,iy)-expo(ix-1,iy) )**2 + &
                  ( expo(ix,iy+1)-expo(ix,iy-1) )**2 ) > gradth) then
           x(ix,iy) = 0
        end if 
      end do
    end do      

    call cp_head (set,setO,"EPIC DETECTION MASK",1)
    call setAttribute(setO, "HDUCLASS", "OGIP    ", &
                            "Format conforms to OGIP/GSFC conventions")
    call setAttribute(setO, "HDUCLAS1", "IMAGE   ", &
                            "File contains an image")
    call setAttribute(setO, "HDUCLAS2", "MASK", &
                            "Detection mask")

    call release(earr) 
    call release(set)

    call release(marr)
    call release(setO) 
         
  end subroutine emask
  
end module emask_mod 
   
